@@ -132,7 +132,10 @@ static void
 queue_flush (GdkWindow *window)
 {
   if (flush_id == 0)
-    flush_id = gdk_threads_add_idle (flush_idle, NULL);
+    {
+      flush_id = gdk_threads_add_idle (flush_idle, NULL);
+      g_source_set_name_by_id (flush_id, "[gtk+] flush_idle");
+    }
 }
 
 static void
@@ -365,16 +368,14 @@ _gdk_broadway_window_destroy (GdkWindow *window,
 				       impl->id);
 }
 
-static cairo_surface_t *
+static gboolean
 gdk_window_broadway_resize_cairo_surface (GdkWindow       *window,
 					  cairo_surface_t *surface,
 					  gint             width,
 					  gint             height)
 {
   /* Image surfaces cannot be resized */
-  cairo_surface_destroy (surface);
-
-  return NULL;
+  return FALSE;
 }
 
 static void
@@ -537,6 +538,19 @@ static void
 gdk_broadway_window_focus (GdkWindow *window,
 			   guint32    timestamp)
 {
+  GdkWindowImplBroadway *impl;
+  GdkBroadwayDisplay *broadway_display;
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  if (GDK_WINDOW_DESTROYED (window) ||
+      !window->accept_focus)
+    return;
+
+  impl = GDK_WINDOW_IMPL_BROADWAY (window->impl);
+  broadway_display = GDK_BROADWAY_DISPLAY (gdk_window_get_display (window));
+  _gdk_broadway_server_window_focus (broadway_display->server,
+				     impl->id);
 }
 
 static void
@@ -696,7 +710,7 @@ gdk_window_broadway_get_geometry (GdkWindow *window,
 
 }
 
-static gint
+static void
 gdk_window_broadway_get_root_coords (GdkWindow *window,
 				     gint       x,
 				     gint       y,
@@ -711,26 +725,6 @@ gdk_window_broadway_get_root_coords (GdkWindow *window,
     *root_x = x + impl->wrapper->x;
   if (root_y)
     *root_y = y + impl->wrapper->y;
-
-  return 1;
-}
-
-static void
-gdk_broadway_window_get_root_origin (GdkWindow *window,
-				     gint      *x,
-				     gint      *y)
-{
-  GdkWindowImplBroadway *impl;
-
-  impl = GDK_WINDOW_IMPL_BROADWAY (window->impl);
-
-  /* TODO: This should take wm frame into account */
-
-  if (x)
-    *x = impl->wrapper->x;
-
-  if (y)
-    *y = impl->wrapper->x;
 }
 
 static void
@@ -1184,6 +1178,34 @@ _gdk_broadway_moveresize_handle_event (GdkDisplay *display,
 
   switch (event->base.type)
     {
+    case BROADWAY_EVENT_TOUCH:
+      if (event->touch.touch_type == 2) /* END */
+        {
+          update_pos (mv_resize,
+                      event->touch.root_x,
+                      event->touch.root_y);
+
+          finish_drag (mv_resize);
+        }
+      else if (event->touch.touch_type == 1) /* UPDATE */
+        {
+          if (mv_resize->moveresize_window->resize_count > 0)
+            {
+              if (mv_resize->moveresize_pending_event)
+                *mv_resize->moveresize_pending_event = *event;
+              else
+                mv_resize->moveresize_pending_event =
+                  g_memdup (event, sizeof (BroadwayInputMsg));
+
+              break;
+            }
+          update_pos (mv_resize,
+                      event->touch.root_x,
+                      event->touch.root_y);
+        }
+
+      break;
+
     case BROADWAY_EVENT_POINTER_MOVE:
       if (mv_resize->moveresize_window->resize_count > 0)
 	{
@@ -1405,10 +1427,32 @@ gdk_broadway_window_begin_move_drag (GdkWindow *window,
 				     gint       root_y,
 				     guint32    timestamp)
 {
+  MoveResizeData *mv_resize;
+  GdkWindowImplBroadway *impl;
+
+  impl = GDK_WINDOW_IMPL_BROADWAY (window->impl);
+
   if (GDK_WINDOW_DESTROYED (window) ||
-      !WINDOW_IS_TOPLEVEL (window))
+      !WINDOW_IS_TOPLEVEL_OR_FOREIGN (window))
     return;
 
+  mv_resize = get_move_resize_data (gdk_window_get_display (window), TRUE);
+
+  mv_resize->is_resize = FALSE;
+  mv_resize->moveresize_button = button;
+  mv_resize->moveresize_x = root_x;
+  mv_resize->moveresize_y = root_y;
+  mv_resize->moveresize_window = g_object_ref (window);
+
+  mv_resize->moveresize_orig_width = gdk_window_get_width (window);
+  mv_resize->moveresize_orig_height = gdk_window_get_height (window);
+
+  mv_resize->moveresize_geom_mask = impl->geometry_hints_mask;
+  mv_resize->moveresize_geometry = impl->geometry_hints;
+
+  calculate_unmoving_origin (mv_resize);
+
+  create_moveresize_window (mv_resize, timestamp);
 }
 
 static gboolean
@@ -1525,7 +1569,6 @@ gdk_window_impl_broadway_class_init (GdkWindowImplBroadwayClass *klass)
   impl_class->set_role = gdk_broadway_window_set_role;
   impl_class->set_startup_id = gdk_broadway_window_set_startup_id;
   impl_class->set_transient_for = gdk_broadway_window_set_transient_for;
-  impl_class->get_root_origin = gdk_broadway_window_get_root_origin;
   impl_class->get_frame_extents = gdk_broadway_window_get_frame_extents;
   impl_class->set_override_redirect = gdk_broadway_window_set_override_redirect;
   impl_class->set_accept_focus = gdk_broadway_window_set_accept_focus;
