@@ -54,13 +54,13 @@
  * Cursors by themselves are not very interesting, they must be be
  * bound to a window for users to see them. This is done with
  * gdk_window_set_cursor() or by setting the cursor member of the
- * #GdkWindowAttr struct passed to gdk_window_new().
+ * #GdkWindowAttr passed to gdk_window_new().
  */
 
 /**
  * GdkCursor:
  *
- * The #GdkCursor structure represents a cursor. Its contents are private.
+ * A #GdkCursor represents a cursor. Its contents are private.
  */
 
 enum {
@@ -153,7 +153,7 @@ gdk_cursor_init (GdkCursor *cursor)
  *
  * Adds a reference to @cursor.
  *
- * Return value: (transfer full): Same @cursor that was passed in
+ * Returns: (transfer full): Same @cursor that was passed in
  *
  * Deprecated: 3.0: Use g_object_ref() instead
  */
@@ -191,7 +191,7 @@ gdk_cursor_unref (GdkCursor *cursor)
  *
  * To make the cursor invisible, use %GDK_BLANK_CURSOR.
  *
- * Return value: a new #GdkCursor
+ * Returns: a new #GdkCursor
  */
 GdkCursor*
 gdk_cursor_new (GdkCursorType cursor_type)
@@ -205,7 +205,7 @@ gdk_cursor_new (GdkCursorType cursor_type)
  *
  * Returns the cursor type for this cursor.
  *
- * Return value: a #GdkCursorType
+ * Returns: a #GdkCursorType
  *
  * Since: 2.22
  **/
@@ -224,64 +224,26 @@ gdk_cursor_get_cursor_type (GdkCursor *cursor)
  *
  * Creates a new cursor from the set of builtin cursors.
  * Some useful ones are:
- * <itemizedlist>
- * <listitem><para>
- *  <inlinegraphic format="PNG" fileref="right_ptr.png"></inlinegraphic> #GDK_RIGHT_PTR (right-facing arrow)
- * </para></listitem>
- * <listitem><para>
- *  <inlinegraphic format="PNG" fileref="crosshair.png"></inlinegraphic> #GDK_CROSSHAIR (crosshair)
- * </para></listitem>
- * <listitem><para>
- *  <inlinegraphic format="PNG" fileref="xterm.png"></inlinegraphic> #GDK_XTERM (I-beam)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="watch.png"></inlinegraphic> #GDK_WATCH (busy)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="fleur.png"></inlinegraphic> #GDK_FLEUR (for moving objects)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="hand1.png"></inlinegraphic> #GDK_HAND1 (a right-pointing hand)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="hand2.png"></inlinegraphic> #GDK_HAND2 (a left-pointing hand)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="left_side.png"></inlinegraphic> #GDK_LEFT_SIDE (resize left side)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="right_side.png"></inlinegraphic> #GDK_RIGHT_SIDE (resize right side)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="top_left_corner.png"></inlinegraphic> #GDK_TOP_LEFT_CORNER (resize northwest corner)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="top_right_corner.png"></inlinegraphic> #GDK_TOP_RIGHT_CORNER (resize northeast corner)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="bottom_left_corner.png"></inlinegraphic> #GDK_BOTTOM_LEFT_CORNER (resize southwest corner)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="bottom_right_corner.png"></inlinegraphic> #GDK_BOTTOM_RIGHT_CORNER (resize southeast corner)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="top_side.png"></inlinegraphic> #GDK_TOP_SIDE (resize top side)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="bottom_side.png"></inlinegraphic> #GDK_BOTTOM_SIDE (resize bottom side)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="sb_h_double_arrow.png"></inlinegraphic> #GDK_SB_H_DOUBLE_ARROW (move vertical splitter)
- * </para></listitem>
- * <listitem><para>
- * <inlinegraphic format="PNG" fileref="sb_v_double_arrow.png"></inlinegraphic> #GDK_SB_V_DOUBLE_ARROW (move horizontal splitter)
- * </para></listitem>
- * <listitem><para>
- * #GDK_BLANK_CURSOR (Blank cursor). Since 2.16
- * </para></listitem>
- * </itemizedlist>
+ * - ![](right_ptr.png) #GDK_RIGHT_PTR (right-facing arrow)
+ * - ![](crosshair.png) #GDK_CROSSHAIR (crosshair)
+ * - ![](xterm.png) #GDK_XTERM (I-beam)
+ * - ![](watch.png) #GDK_WATCH (busy)
+ * - ![](fleur.png) #GDK_FLEUR (for moving objects)
+ * - ![](hand1.png) #GDK_HAND1 (a right-pointing hand)
+ * - ![](hand2.png) #GDK_HAND2 (a left-pointing hand)
+ * - ![](left_side.png) #GDK_LEFT_SIDE (resize left side)
+ * - ![](right_side.png) #GDK_RIGHT_SIDE (resize right side)
+ * - ![](top_left_corner.png) #GDK_TOP_LEFT_CORNER (resize northwest corner)
+ * - ![](top_right_corner.png) #GDK_TOP_RIGHT_CORNER (resize northeast corner)
+ * - ![](bottom_left_corner.png) #GDK_BOTTOM_LEFT_CORNER (resize southwest corner)
+ * - ![](bottom_right_corner.png) #GDK_BOTTOM_RIGHT_CORNER (resize southeast corner)
+ * - ![](top_side.png) #GDK_TOP_SIDE (resize top side)
+ * - ![](bottom_side.png) #GDK_BOTTOM_SIDE (resize bottom side)
+ * - ![](sb_h_double_arrow.png) #GDK_SB_H_DOUBLE_ARROW (move vertical splitter)
+ * - ![](sb_v_double_arrow.png) #GDK_SB_V_DOUBLE_ARROW (move horizontal splitter)
+ * - #GDK_BLANK_CURSOR (Blank cursor). Since 2.16
  *
- * Return value: a new #GdkCursor
+ * Returns: a new #GdkCursor
  *
  * Since: 2.2
  **/
@@ -320,8 +282,8 @@ gdk_cursor_new_from_name (GdkDisplay  *display,
  * gdk_cursor_new_from_pixbuf:
  * @display: the #GdkDisplay for which the cursor will be created
  * @pixbuf: the #GdkPixbuf containing the cursor image
- * @x: the horizontal offset of the 'hotspot' of the cursor.
- * @y: the vertical offset of the 'hotspot' of the cursor.
+ * @x: the horizontal offset of the “hotspot” of the cursor.
+ * @y: the vertical offset of the “hotspot” of the cursor.
  *
  * Creates a new cursor from a pixbuf.
  *
@@ -334,9 +296,9 @@ gdk_cursor_new_from_name (GdkDisplay  *display,
  * gdk_display_get_maximal_cursor_size() give information about
  * cursor sizes.
  *
- * If @x or @y are <literal>-1</literal>, the pixbuf must have
- * options named "x_hot" and "y_hot", resp., containing
- * integer values between %0 and the width resp. height of
+ * If @x or @y are `-1`, the pixbuf must have
+ * options named “x_hot” and “y_hot”, resp., containing
+ * integer values between `0` and the width resp. height of
  * the pixbuf. (Since: 3.0)
  *
  * On the X backend, support for RGBA cursors requires a
@@ -356,6 +318,7 @@ gdk_cursor_new_from_pixbuf (GdkDisplay *display,
   const char *option;
   char *end;
   gint64 value;
+  GdkCursor *cursor;
  
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
   g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
@@ -384,15 +347,19 @@ gdk_cursor_new_from_pixbuf (GdkDisplay *display,
 
   surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, 1, NULL);
   
-  return GDK_DISPLAY_GET_CLASS (display)->get_cursor_for_surface (display, surface, x, y);
+  cursor = GDK_DISPLAY_GET_CLASS (display)->get_cursor_for_surface (display, surface, x, y);
+
+  cairo_surface_destroy (surface);
+
+  return cursor;
 }
 
 /**
  * gdk_cursor_new_from_surface:
  * @display: the #GdkDisplay for which the cursor will be created
  * @surface: the cairo image surface containing the cursor pixel data
- * @x: the horizontal offset of the 'hotspot' of the cursor
- * @y: the vertical offset of the 'hotspot' of the cursor
+ * @x: the horizontal offset of the “hotspot” of the cursor
+ * @y: the vertical offset of the “hotspot” of the cursor
  *
  * Creates a new cursor from a cairo image surface.
  *
