@@ -188,6 +188,7 @@ gdk_wayland_device_update_window_cursor (GdkWaylandDeviceData *wd)
       id = g_timeout_add (next_image_delay,
                           (GSourceFunc)gdk_wayland_device_update_window_cursor,
                           wd);
+      g_source_set_name_by_id (id, "[gtk+] gdk_wayland_device_update_window_cursor");
 
       wd->cursor_timeout_id = id;
       wd->cursor_image_index = next_image_index;
@@ -236,6 +237,37 @@ gdk_wayland_device_warp (GdkDevice *device,
 {
 }
 
+static void
+get_coordinates (GdkWaylandDeviceData *data,
+                 double *x, double *y,
+                 double *x_root, double *y_root)
+{
+  int root_x, root_y;
+
+  if (x)
+    *x = data->surface_x;
+  if (y)
+    *y = data->surface_y;
+
+  if (data->pointer_focus)
+    {
+      gdk_window_get_root_coords (data->pointer_focus,
+                                  data->surface_x,
+                                  data->surface_y,
+                                  &root_x, &root_y);
+    }
+  else
+    {
+      root_x = data->surface_x;
+      root_y = data->surface_y;
+    }
+
+  if (x_root)
+    *x_root = root_x;
+  if (y_root)
+    *y_root = root_y;
+}
+
 static void
 gdk_wayland_device_query_state (GdkDevice        *device,
                                 GdkWindow        *window,
@@ -257,17 +289,10 @@ gdk_wayland_device_query_state (GdkDevice        *device,
     *root_window = gdk_screen_get_root_window (default_screen);
   if (child_window)
     *child_window = wd->pointer_focus;
-  /* TODO: Do something clever for relative here */
-  if (root_x)
-    *root_x = wd->surface_x;
-  if (root_y)
-    *root_y = wd->surface_y;
-  if (win_x)
-    *win_x = wd->surface_x;
-  if (win_y)
-    *win_y = wd->surface_y;
   if (mask)
     *mask = wd->modifiers;
+
+  get_coordinates (wd, win_x, win_y, root_x, root_y);
 }
 
 static GdkGrabStatus
@@ -613,8 +638,6 @@ static const struct wl_data_device_listener data_device_listener = {
   data_device_selection
 };
 
-
-
 static void
 pointer_handle_enter (void              *data,
                       struct wl_pointer *pointer,
@@ -638,22 +661,25 @@ pointer_handle_enter (void              *data,
   device->pointer_focus = wl_surface_get_user_data(surface);
   g_object_ref(device->pointer_focus);
 
+  device->surface_x = wl_fixed_to_double (sx);
+  device->surface_y = wl_fixed_to_double (sy);
+  device->enter_serial = serial;
+
   event = gdk_event_new (GDK_ENTER_NOTIFY);
   event->crossing.window = g_object_ref (device->pointer_focus);
   gdk_event_set_device (event, device->pointer);
   event->crossing.subwindow = NULL;
   event->crossing.time = (guint32)(g_get_monotonic_time () / 1000);
-  event->crossing.x = wl_fixed_to_double (sx);
-  event->crossing.y = wl_fixed_to_double (sy);
-
   event->crossing.mode = GDK_CROSSING_NORMAL;
   event->crossing.detail = GDK_NOTIFY_ANCESTOR;
   event->crossing.focus = TRUE;
   event->crossing.state = 0;
 
-  device->surface_x = wl_fixed_to_double (sx);
-  device->surface_y = wl_fixed_to_double (sy);
-  device->enter_serial = serial;
+  get_coordinates (device,
+                   &event->crossing.x,
+                   &event->crossing.y,
+                   &event->crossing.x_root,
+                   &event->crossing.y_root);
 
   _gdk_wayland_display_deliver_event (device->display, event);
 
@@ -687,14 +713,17 @@ pointer_handle_leave (void              *data,
   gdk_event_set_device (event, device->pointer);
   event->crossing.subwindow = NULL;
   event->crossing.time = (guint32)(g_get_monotonic_time () / 1000);
-  event->crossing.x = device->surface_x;
-  event->crossing.y = device->surface_y;
-
   event->crossing.mode = GDK_CROSSING_NORMAL;
   event->crossing.detail = GDK_NOTIFY_ANCESTOR;
   event->crossing.focus = TRUE;
   event->crossing.state = 0;
 
+  get_coordinates (device,
+                   &event->crossing.x,
+                   &event->crossing.y,
+                   &event->crossing.x_root,
+                   &event->crossing.y_root);
+
   _gdk_wayland_display_deliver_event (device->display, event);
 
   GDK_NOTE (EVENTS,
@@ -736,13 +765,17 @@ pointer_handle_motion (void              *data,
   event->motion.window = g_object_ref (device->pointer_focus);
   gdk_event_set_device (event, device->pointer);
   event->motion.time = time;
-  event->motion.x = wl_fixed_to_double (sx);
-  event->motion.y = wl_fixed_to_double (sy);
   event->motion.axes = NULL;
   event->motion.state = device->modifiers;
   event->motion.is_hint = 0;
   gdk_event_set_screen (event, display->screen);
 
+  get_coordinates (device,
+                   &event->motion.x,
+                   &event->motion.y,
+                   &event->motion.x_root,
+                   &event->motion.y_root);
+
   GDK_NOTE (EVENTS,
             g_message ("motion %d %d, state %d",
                        sx, sy, event->button.state));
@@ -791,13 +824,17 @@ pointer_handle_button (void              *data,
   event->button.window = g_object_ref (device->pointer_focus);
   gdk_event_set_device (event, device->pointer);
   event->button.time = time;
-  event->button.x = device->surface_x;
-  event->button.y = device->surface_y;
   event->button.axes = NULL;
   event->button.state = device->modifiers;
   event->button.button = gdk_button;
   gdk_event_set_screen (event, display->screen);
 
+  get_coordinates (device,
+                   &event->button.x,
+                   &event->button.y,
+                   &event->button.x_root,
+                   &event->button.y_root);
+
   modifier = 1 << (8 + gdk_button - 1);
   if (state)
     device->modifiers |= modifier;
@@ -846,14 +883,18 @@ pointer_handle_axis (void              *data,
   event->scroll.window = g_object_ref (device->pointer_focus);
   gdk_event_set_device (event, device->pointer);
   event->scroll.time = time;
-  event->scroll.x = (gdouble) device->surface_x;
-  event->scroll.y = (gdouble) device->surface_y;
   event->scroll.direction = GDK_SCROLL_SMOOTH;
   event->scroll.delta_x = delta_x;
   event->scroll.delta_y = delta_y;
   event->scroll.state = device->modifiers;
   gdk_event_set_screen (event, display->screen);
 
+  get_coordinates (device,
+                   &event->scroll.x,
+                   &event->scroll.y,
+                   &event->scroll.x_root,
+                   &event->scroll.y_root);
+
   GDK_NOTE (EVENTS,
             g_message ("scroll %f %f",
                        event->scroll.delta_x, event->scroll.delta_y));
@@ -910,8 +951,6 @@ keyboard_handle_enter (void               *data,
                        device, device->keyboard_focus));
 
   _gdk_wayland_display_deliver_event (device->display, event);
-
-  _gdk_wayland_window_add_focus (device->keyboard_focus);
 }
 
 static void
@@ -934,7 +973,6 @@ keyboard_handle_leave (void               *data,
 
   _gdk_wayland_display_update_serial (wayland_display, serial);
 
-  _gdk_wayland_window_remove_focus (device->keyboard_focus);
   event = gdk_event_new (GDK_FOCUS_CHANGE);
   event->focus_change.window = g_object_ref (device->keyboard_focus);
   event->focus_change.send_event = FALSE;
@@ -1112,10 +1150,12 @@ deliver_key_event (GdkWaylandDeviceData *device,
 
       device->repeat_timer =
         gdk_threads_add_timeout (delay, keyboard_repeat, device);
+      g_source_set_name_by_id (device->repeat_timer, "[gtk+] keyboard_repeat");
       return TRUE;
     case 2:
       device->repeat_timer =
         gdk_threads_add_timeout (interval, keyboard_repeat, device);
+      g_source_set_name_by_id (device->repeat_timer, "[gtk+] keyboard_repeat");
       return FALSE;
     default:
       return TRUE;
