@@ -17,8 +17,6 @@
 
 #include "config.h"
 
-#include <wayland-egl.h>
-
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -36,17 +34,52 @@
 #include "gdkkeysprivate.h"
 #include "gdkprivate-wayland.h"
 
-G_DEFINE_TYPE (GdkDisplayWayland, _gdk_display_wayland, GDK_TYPE_DISPLAY)
+/**
+ * SECTION:wayland_interaction
+ * @Short_description: Wayland backend-specific functions
+ * @Title: Wayland Interaction
+ *
+ * The functions in this section are specific to the GDK Wayland backend.
+ * To use them, you need to include the `<gdk/gdkwayland.h>` header and use
+ * the Wayland-specific pkg-config files to build your application (either
+ * `gdk-wayland-3.0` or `gtk+-wayland-3.0`).
+ *
+ * To make your code compile with other GDK backends, guard backend-specific
+ * calls by an ifdef as follows. Since GDK may be built with multiple
+ * backends, you should also check for the backend that is in use (e.g. by
+ * using the GDK_IS_WAYLAND_DISPLAY() macro).
+ * |[<!-- language="C" -->
+ * #ifdef GDK_WINDOWING_WAYLAND
+ *   if (GDK_IS_WAYLAND_DISPLAY (display))
+ *     {
+ *       // make Wayland-specific calls here
+ *     }
+ *   else
+ * #endif
+ * #ifdef GDK_WINDOWING_X11
+ *   if (GDK_IS_X11_DISPLAY (display))
+ *     {
+ *       // make X11-specific calls here
+ *     }
+ *   else
+ * #endif
+ *   g_error ("Unsupported GDK backend");
+ * ]|
+ */
+
+static void _gdk_wayland_display_load_cursor_theme (GdkWaylandDisplay *wayland_display);
+
+G_DEFINE_TYPE (GdkWaylandDisplay, gdk_wayland_display, GDK_TYPE_DISPLAY)
 
 static void
 gdk_input_init (GdkDisplay *display)
 {
-  GdkDisplayWayland *display_wayland;
+  GdkWaylandDisplay *display_wayland;
   GdkDeviceManager *device_manager;
   GdkDevice *device;
   GList *list, *l;
 
-  display_wayland = GDK_DISPLAY_WAYLAND (display);
+  display_wayland = GDK_WAYLAND_DISPLAY (display);
   device_manager = gdk_display_get_device_manager (display);
 
   /* For backwards compatibility, just add
@@ -73,7 +106,7 @@ gdk_input_init (GdkDisplay *display)
 
   for (l = list; l; l = l->next)
     {
-      device = list->data;
+      device = l->data;
 
       if (gdk_device_get_source (device) != GDK_SOURCE_MOUSE)
 	continue;
@@ -89,116 +122,119 @@ gdk_input_init (GdkDisplay *display)
 }
 
 static void
-output_handle_geometry(void *data,
-		       struct wl_output *wl_output,
-		       int x, int y, int physical_width, int physical_height,
-		       int subpixel, const char *make, const char *model)
-{
-  /*
-    g_signal_emit_by_name (screen, "monitors-changed");
-    g_signal_emit_by_name (screen, "size-changed");
-  */
-}
-static void
-display_handle_mode(void *data,
-                   struct wl_output *wl_output,
-                   uint32_t flags,
-                   int width,
-                   int height,
-                   int refresh)
+init_sync_callback(void *data, struct wl_callback *callback, uint32_t serial)
 {
+  GdkWaylandDisplay *display = data;
+
+  display->init_ref_count--;
+  wl_callback_destroy(callback);
 }
 
-static const struct wl_output_listener output_listener = {
-	output_handle_geometry,
-	display_handle_mode
+static const struct wl_callback_listener init_sync_listener = {
+	init_sync_callback
 };
 
 static void
-gdk_display_handle_global(struct wl_display *display, uint32_t id,
-			  const char *interface, uint32_t version, void *data)
+wait_for_roundtrip(GdkWaylandDisplay *display)
 {
-  GdkDisplayWayland *display_wayland = data;
+  struct wl_callback *callback;
+
+  display->init_ref_count++;
+  callback = wl_display_sync(display->wl_display);
+  wl_callback_add_listener(callback, &init_sync_listener, display);
+}
+
+static void
+xdg_shell_ping (void             *data,
+                struct xdg_shell *xdg_shell,
+                uint32_t          serial)
+{
+  GdkWaylandDisplay *wayland_display = data;
+
+  _gdk_wayland_display_update_serial (wayland_display, serial);
+
+  xdg_shell_pong (xdg_shell, serial);
+}
+
+static const struct xdg_shell_listener xdg_shell_listener = {
+  xdg_shell_ping,
+};
+
+static void
+gdk_registry_handle_global(void *data, struct wl_registry *registry, uint32_t id,
+					const char *interface, uint32_t version)
+{
+  GdkWaylandDisplay *display_wayland = data;
   GdkDisplay *gdk_display = GDK_DISPLAY_OBJECT (data);
-  struct wl_input_device *input;
+  struct wl_seat *seat;
+  struct wl_output *output;
 
   if (strcmp(interface, "wl_compositor") == 0) {
     display_wayland->compositor =
-      wl_display_bind(display, id, &wl_compositor_interface);
+      wl_registry_bind(display_wayland->wl_registry, id, &wl_compositor_interface, MIN (version, 3));
+    display_wayland->compositor_version = MIN (version, 3);
   } else if (strcmp(interface, "wl_shm") == 0) {
-    display_wayland->shm = wl_display_bind(display, id, &wl_shm_interface);
-  } else if (strcmp(interface, "wl_shell") == 0) {
-    display_wayland->shell = wl_display_bind(display, id, &wl_shell_interface);
+   display_wayland->shm =
+	wl_registry_bind(display_wayland->wl_registry, id, &wl_shm_interface, 1);
+
+    /* SHM interface is prerequisite */
+    _gdk_wayland_display_load_cursor_theme(display_wayland);
+  } else if (strcmp(interface, "xdg_shell") == 0) {
+    display_wayland->xdg_shell =
+      wl_registry_bind(display_wayland->wl_registry, id, &xdg_shell_interface, 1);
+    xdg_shell_use_unstable_version(display_wayland->xdg_shell, XDG_SHELL_VERSION_CURRENT);
+    xdg_shell_add_listener(display_wayland->xdg_shell, &xdg_shell_listener, display_wayland);
+  } else if (strcmp(interface, "gtk_shell") == 0) {
+    display_wayland->gtk_shell =
+      wl_registry_bind(display_wayland->wl_registry, id, &gtk_shell_interface, 1);
+    _gdk_wayland_screen_set_has_gtk_shell (display_wayland->screen);
+    /* We need another roundtrip to receive the shell capabilities */
+    wait_for_roundtrip(display_wayland);
   } else if (strcmp(interface, "wl_output") == 0) {
-    display_wayland->output =
-      wl_display_bind(display, id, &wl_output_interface);
-    wl_output_add_listener(display_wayland->output,
-			   &output_listener, display_wayland);
-  } else if (strcmp(interface, "wl_input_device") == 0) {
-    input = wl_display_bind(display, id, &wl_input_device_interface);
-    _gdk_wayland_device_manager_add_device (gdk_display->device_manager,
-					    input);
+    output =
+      wl_registry_bind(display_wayland->wl_registry, id, &wl_output_interface, MIN (version, 2));
+    _gdk_wayland_screen_add_output(display_wayland->screen, id, output, MIN (version, 2));
+    /* We need another roundtrip to receive the modes and geometry
+     * events for the output, which gives us the physical properties
+     * and available modes on the output. */
+    wait_for_roundtrip(display_wayland);
+  } else if (strcmp(interface, "wl_seat") == 0) {
+    seat = wl_registry_bind(display_wayland->wl_registry, id, &wl_seat_interface, 1);
+    _gdk_wayland_device_manager_add_seat (gdk_display->device_manager, id, seat);
+    /* We need another roundtrip to receive the wl_seat capabilities
+     * event which informs us of available input devices on this
+     * seat. */
+    wait_for_roundtrip(display_wayland);
   } else if (strcmp(interface, "wl_data_device_manager") == 0) {
       display_wayland->data_device_manager =
-        wl_display_bind(display, id,
-                        &wl_data_device_manager_interface);
+        wl_registry_bind(display_wayland->wl_registry, id,
+					&wl_data_device_manager_interface, 1);
   }
 }
 
-static gboolean
-gdk_display_init_egl(GdkDisplay *display)
+static void
+gdk_registry_handle_global_remove(void               *data,
+                                  struct wl_registry *registry,
+                                  uint32_t            id)
 {
-  GdkDisplayWayland *display_wayland = GDK_DISPLAY_WAYLAND (display);
-  EGLint major, minor, i;
-  void *p;
+  GdkWaylandDisplay *display_wayland = data;
+  GdkDisplay *display = GDK_DISPLAY (display_wayland);
 
-  static const struct { const char *f; unsigned int offset; }
-  extension_functions[] = {
-    { "glEGLImageTargetTexture2DOES", offsetof(GdkDisplayWayland, image_target_texture_2d) },
-    { "eglCreateImageKHR", offsetof(GdkDisplayWayland, create_image) },
-    { "eglDestroyImageKHR", offsetof(GdkDisplayWayland, destroy_image) }
-  };
+  _gdk_wayland_device_manager_remove_seat (display->device_manager, id);
+  _gdk_wayland_screen_remove_output (display_wayland->screen, id);
 
-  display_wayland->egl_display =
-    eglGetDisplay(display_wayland->wl_display);
-  if (!eglInitialize(display_wayland->egl_display, &major, &minor)) {
-    fprintf(stderr, "failed to initialize display\n");
-    return FALSE;
-  }
+  /* FIXME: the object needs to be destroyed here, we're leaking */
+}
 
-  eglBindAPI(EGL_OPENGL_API);
+static const struct wl_registry_listener registry_listener = {
+    gdk_registry_handle_global,
+    gdk_registry_handle_global_remove
+};
 
-  display_wayland->egl_context =
-    eglCreateContext(display_wayland->egl_display, NULL, EGL_NO_CONTEXT, NULL);
-  if (display_wayland->egl_context == NULL) {
-    fprintf(stderr, "failed to create context\n");
-    return FALSE;
-  }
-
-  if (!eglMakeCurrent(display_wayland->egl_display,
-		      NULL, NULL, display_wayland->egl_context)) {
-    fprintf(stderr, "faile to make context current\n");
-    return FALSE;
-  }
-
-  display_wayland->cairo_device =
-    cairo_egl_device_create(display_wayland->egl_display,
-			    display_wayland->egl_context);
-  if (cairo_device_status (display_wayland->cairo_device) != CAIRO_STATUS_SUCCESS) {
-    fprintf(stderr, "failed to get cairo drm device\n");
-    return FALSE;
-  }
-
-  for (i = 0; i < G_N_ELEMENTS(extension_functions); i++) {
-    p = eglGetProcAddress(extension_functions[i].f);
-    *(void **) ((char *) display_wayland + extension_functions[i].offset) = p;
-    if (p == NULL) {
-      fprintf(stderr, "failed to look up %s\n", extension_functions[i].f);
-      return FALSE;
-    }
-  }
-
-  return TRUE;
+static void
+log_handler(const char *format, va_list args)
+{
+  g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, format, args);
 }
 
 GdkDisplay *
@@ -206,14 +242,16 @@ _gdk_wayland_display_open (const gchar *display_name)
 {
   struct wl_display *wl_display;
   GdkDisplay *display;
-  GdkDisplayWayland *display_wayland;
+  GdkWaylandDisplay *display_wayland;
+
+  wl_log_set_handler_client(log_handler);
 
   wl_display = wl_display_connect(display_name);
   if (!wl_display)
     return NULL;
 
-  display = g_object_new (GDK_TYPE_DISPLAY_WAYLAND, NULL);
-  display_wayland = GDK_DISPLAY_WAYLAND (display);
+  display = g_object_new (GDK_TYPE_WAYLAND_DISPLAY, NULL);
+  display_wayland = GDK_WAYLAND_DISPLAY (display);
 
   display_wayland->wl_display = wl_display;
 
@@ -222,10 +260,14 @@ _gdk_wayland_display_open (const gchar *display_name)
   display->device_manager = _gdk_wayland_device_manager_new (display);
 
   /* Set up listener so we'll catch all events. */
-  wl_display_add_global_listener(display_wayland->wl_display,
-				 gdk_display_handle_global, display_wayland);
+  display_wayland->wl_registry = wl_display_get_registry(display_wayland->wl_display);
+  wl_registry_add_listener(display_wayland->wl_registry, &registry_listener, display_wayland);
 
-  gdk_display_init_egl(display);
+  /* We use init_ref_count to track whether some part of our
+   * initialization still needs a roundtrip to complete. */
+  wait_for_roundtrip(display_wayland);
+  while (display_wayland->init_ref_count > 0)
+    wl_display_roundtrip(display_wayland->wl_display);
 
   display_wayland->event_source =
     _gdk_wayland_display_event_source_new (display);
@@ -233,7 +275,6 @@ _gdk_wayland_display_open (const gchar *display_name)
   gdk_input_init (display);
 
   g_signal_emit_by_name (display, "opened");
-  g_signal_emit_by_name (gdk_display_manager_get(), "display_opened", display);
 
   return display;
 }
@@ -241,10 +282,8 @@ _gdk_wayland_display_open (const gchar *display_name)
 static void
 gdk_wayland_display_dispose (GObject *object)
 {
-  GdkDisplayWayland *display_wayland = GDK_DISPLAY_WAYLAND (object);
+  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (object);
 
-  _gdk_wayland_display_manager_remove_display (gdk_display_manager_get (),
-					       GDK_DISPLAY (display_wayland));
   g_list_foreach (display_wayland->input_devices,
 		  (GFunc) g_object_run_dispose, NULL);
 
@@ -257,19 +296,15 @@ gdk_wayland_display_dispose (GObject *object)
       display_wayland->event_source = NULL;
     }
 
-  eglTerminate(display_wayland->egl_display);
-
-  G_OBJECT_CLASS (_gdk_display_wayland_parent_class)->dispose (object);
+  G_OBJECT_CLASS (gdk_wayland_display_parent_class)->dispose (object);
 }
 
 static void
 gdk_wayland_display_finalize (GObject *object)
 {
-  GdkDisplayWayland *display_wayland = GDK_DISPLAY_WAYLAND (object);
+  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (object);
 
-  /* Keymap */
-  if (display_wayland->keymap)
-    g_object_unref (display_wayland->keymap);
+  _gdk_wayland_display_finalize_cursors (display_wayland);
 
   /* input GdkDevice list */
   g_list_free_full (display_wayland->input_devices, g_object_unref);
@@ -278,7 +313,7 @@ gdk_wayland_display_finalize (GObject *object)
 
   g_free (display_wayland->startup_notification_id);
 
-  G_OBJECT_CLASS (_gdk_display_wayland_parent_class)->finalize (object);
+  G_OBJECT_CLASS (gdk_wayland_display_parent_class)->finalize (object);
 }
 
 static const gchar *
@@ -287,28 +322,12 @@ gdk_wayland_display_get_name (GdkDisplay *display)
   return "Wayland";
 }
 
-static gint
-gdk_wayland_display_get_n_screens (GdkDisplay *display)
-{
-  return 1;
-}
-
-static GdkScreen *
-gdk_wayland_display_get_screen (GdkDisplay *display, 
-				gint        screen_num)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  g_return_val_if_fail (screen_num == 0, NULL);
-
-  return GDK_DISPLAY_WAYLAND (display)->screen;
-}
-
 static GdkScreen *
 gdk_wayland_display_get_default_screen (GdkDisplay *display)
 {
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
 
-  return GDK_DISPLAY_WAYLAND (display)->screen;
+  return GDK_WAYLAND_DISPLAY (display)->screen;
 }
 
 static void
@@ -320,11 +339,11 @@ gdk_wayland_display_beep (GdkDisplay *display)
 static void
 gdk_wayland_display_sync (GdkDisplay *display)
 {
-  GdkDisplayWayland *display_wayland;
+  GdkWaylandDisplay *display_wayland;
 
   g_return_if_fail (GDK_IS_DISPLAY (display));
 
-  display_wayland = GDK_DISPLAY_WAYLAND (display);
+  display_wayland = GDK_WAYLAND_DISPLAY (display);
 
   wl_display_roundtrip(display_wayland->wl_display);
 }
@@ -335,8 +354,7 @@ gdk_wayland_display_flush (GdkDisplay *display)
   g_return_if_fail (GDK_IS_DISPLAY (display));
 
   if (!display->closed)
-    _gdk_wayland_display_flush (display,
-				GDK_DISPLAY_WAYLAND (display)->event_source);
+    wl_display_flush(GDK_WAYLAND_DISPLAY (display)->wl_display);
 }
 
 static gboolean
@@ -386,7 +404,7 @@ gdk_wayland_display_store_clipboard (GdkDisplay    *display,
 static gboolean
 gdk_wayland_display_supports_shapes (GdkDisplay *display)
 {
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -398,7 +416,7 @@ gdk_wayland_display_supports_input_shapes (GdkDisplay *display)
 static gboolean
 gdk_wayland_display_supports_composite (GdkDisplay *display)
 {
-  return TRUE;
+  return FALSE;
 }
 
 static GList *
@@ -406,7 +424,7 @@ gdk_wayland_display_list_devices (GdkDisplay *display)
 {
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
 
-  return GDK_DISPLAY_WAYLAND (display)->input_devices;
+  return GDK_WAYLAND_DISPLAY (display)->input_devices;
 }
 
 static void
@@ -427,11 +445,6 @@ gdk_wayland_display_get_next_serial (GdkDisplay *display)
   return ++serial;
 }
 
-void
-_gdk_wayland_display_make_default (GdkDisplay *display)
-{
-}
-
 /**
  * gdk_wayland_display_broadcast_startup_message:
  * @display: a #GdkDisplay
@@ -446,10 +459,9 @@ _gdk_wayland_display_make_default (GdkDisplay *display)
  *
  * This is a convenience function for use by code that implements the
  * freedesktop startup notification specification. Applications should
- * not normally need to call it directly. See the <ulink
- * url="http://standards.freedesktop.org/startup-notification-spec/startup-notification-latest.txt">Startup
- * Notification Protocol specification</ulink> for
- * definitions of the message types and keys that can be used.
+ * not normally need to call it directly. See the 
+ * [Startup Notification Protocol specification](http://standards.freedesktop.org/startup-notification-spec/startup-notification-latest.txt)
+ * for definitions of the message types and keys that can be used.
  *
  * Since: 2.12
  **/
@@ -490,8 +502,6 @@ gdk_wayland_display_broadcast_startup_message (GdkDisplay *display,
     }
   va_end (ap);
 
-  printf ("startup message: %s\n", message->str);
-
   g_string_free (message, TRUE);
 }
 
@@ -517,18 +527,43 @@ gdk_wayland_display_event_data_free (GdkDisplay *display,
 {
 }
 
-static GdkKeymap *
-gdk_wayland_display_get_keymap (GdkDisplay *display)
+GdkKeymap *
+_gdk_wayland_display_get_keymap (GdkDisplay *display)
 {
-  GdkDisplayWayland *display_wayland;
+  GdkDeviceManager *device_manager;
+  GList *list, *l;
+  GdkDevice *core_keyboard = NULL;
+  static GdkKeymap *tmp_keymap = NULL;
 
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  display_wayland = GDK_DISPLAY_WAYLAND (display);
+  device_manager = gdk_display_get_device_manager (display);
+  list = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
 
-  if (!display_wayland->keymap)
-    display_wayland->keymap = _gdk_wayland_keymap_new (display);
+  for (l = list; l; l = l->next)
+    {
+      GdkDevice *device;
+      device = list->data;
 
-  return display_wayland->keymap;
+      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
+	continue;
+
+      core_keyboard = device;
+      break;
+    }
+
+  g_list_free (list);
+
+  if (core_keyboard && tmp_keymap)
+    {
+      g_object_unref (tmp_keymap);
+      tmp_keymap = NULL;
+    }
+
+  if (core_keyboard)
+    return _gdk_wayland_device_get_keymap (core_keyboard);
+
+  tmp_keymap = _gdk_wayland_keymap_new ();
+
+  return tmp_keymap;
 }
 
 static void
@@ -544,7 +579,7 @@ gdk_wayland_display_pop_error_trap (GdkDisplay *display,
 }
 
 static void
-_gdk_display_wayland_class_init (GdkDisplayWaylandClass * class)
+gdk_wayland_display_class_init (GdkWaylandDisplayClass * class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   GdkDisplayClass *display_class = GDK_DISPLAY_CLASS (class);
@@ -552,10 +587,8 @@ _gdk_display_wayland_class_init (GdkDisplayWaylandClass * class)
   object_class->dispose = gdk_wayland_display_dispose;
   object_class->finalize = gdk_wayland_display_finalize;
 
-  display_class->window_type = _gdk_wayland_window_get_type ();
+  display_class->window_type = gdk_wayland_window_get_type ();
   display_class->get_name = gdk_wayland_display_get_name;
-  display_class->get_n_screens = gdk_wayland_display_get_n_screens;
-  display_class->get_screen = gdk_wayland_display_get_screen;
   display_class->get_default_screen = gdk_wayland_display_get_default_screen;
   display_class->beep = gdk_wayland_display_beep;
   display_class->sync = gdk_wayland_display_sync;
@@ -576,7 +609,7 @@ _gdk_display_wayland_class_init (GdkDisplayWaylandClass * class)
   display_class->get_maximal_cursor_size = _gdk_wayland_display_get_maximal_cursor_size;
   display_class->get_cursor_for_type = _gdk_wayland_display_get_cursor_for_type;
   display_class->get_cursor_for_name = _gdk_wayland_display_get_cursor_for_name;
-  display_class->get_cursor_for_pixbuf = _gdk_wayland_display_get_cursor_for_pixbuf;
+  display_class->get_cursor_for_surface = _gdk_wayland_display_get_cursor_for_surface;
   display_class->supports_cursor_alpha = _gdk_wayland_display_supports_cursor_alpha;
   display_class->supports_cursor_color = _gdk_wayland_display_supports_cursor_color;
   display_class->before_process_all_updates = gdk_wayland_display_before_process_all_updates;
@@ -586,7 +619,7 @@ _gdk_display_wayland_class_init (GdkDisplayWaylandClass * class)
   display_class->event_data_copy = gdk_wayland_display_event_data_copy;
   display_class->event_data_free = gdk_wayland_display_event_data_free;
   display_class->create_window_impl = _gdk_wayland_display_create_window_impl;
-  display_class->get_keymap = gdk_wayland_display_get_keymap;
+  display_class->get_keymap = _gdk_wayland_display_get_keymap;
   display_class->push_error_trap = gdk_wayland_display_push_error_trap;
   display_class->pop_error_trap = gdk_wayland_display_pop_error_trap;
   display_class->get_selection_owner = _gdk_wayland_display_get_selection_owner;
@@ -599,8 +632,133 @@ _gdk_display_wayland_class_init (GdkDisplayWaylandClass * class)
 }
 
 static void
-_gdk_display_wayland_init (GdkDisplayWayland *display)
+gdk_wayland_display_init (GdkWaylandDisplay *display)
 {
-  _gdk_wayland_display_manager_add_display (gdk_display_manager_get (),
-					    GDK_DISPLAY (display));
+  display->xkb_context = xkb_context_new (0);
+}
+
+void
+gdk_wayland_display_set_cursor_theme (GdkDisplay  *display,
+                                      const gchar *name,
+                                      gint         size)
+{
+  GdkWaylandDisplay *wayland_display = GDK_WAYLAND_DISPLAY(display);
+  struct wl_cursor_theme *theme;
+
+  g_assert (wayland_display);
+  g_assert (wayland_display->shm);
+
+  theme = wl_cursor_theme_load (name, size, wayland_display->shm);
+  if (theme == NULL)
+    {
+      g_warning ("Failed to load cursor theme %s\n", name);
+      return;
+    }
+
+  _gdk_wayland_display_update_cursors (wayland_display, theme);
+
+  if (wayland_display->cursor_theme != NULL)
+    wl_cursor_theme_destroy (wayland_display->cursor_theme);
+  wayland_display->cursor_theme = theme;
+}
+
+static void
+_gdk_wayland_display_load_cursor_theme (GdkWaylandDisplay *wayland_display)
+{
+  guint size;
+  const gchar *name;
+  GValue v = G_VALUE_INIT;
+
+  g_assert (wayland_display);
+  g_assert (wayland_display->shm);
+
+  g_value_init (&v, G_TYPE_INT);
+  if (gdk_screen_get_setting (wayland_display->screen, "gtk-cursor-theme-size", &v))
+    size = g_value_get_int (&v);
+  else
+    size = 32;
+  g_value_unset (&v);
+
+  g_value_init (&v, G_TYPE_STRING);
+  if (gdk_screen_get_setting (wayland_display->screen, "gtk-cursor-theme-name", &v))
+    name = g_value_get_string (&v);
+  else
+    name = "default";
+
+  gdk_wayland_display_set_cursor_theme (GDK_DISPLAY (wayland_display),
+                                        name, size);
+  g_value_unset (&v);
+}
+
+guint32
+_gdk_wayland_display_get_serial (GdkWaylandDisplay *wayland_display)
+{
+  return wayland_display->serial;
+}
+
+void
+_gdk_wayland_display_update_serial (GdkWaylandDisplay *wayland_display,
+                                    guint32            serial)
+{
+  wayland_display->serial = serial;
+}
+
+/**
+ * gdk_wayland_display_get_wl_display
+ * @display: (type GdkWaylandDisplay): a #GdkDisplay
+ *
+ * Returns the Wayland wl_display of a #GdkDisplay
+ *
+ * Returns: (transfer none): a Wayland wl_display
+ *
+ * Since: 3.8
+ */
+struct wl_display *
+gdk_wayland_display_get_wl_display(GdkDisplay *display)
+{
+  GdkWaylandDisplay *wayland_display = GDK_WAYLAND_DISPLAY(display);
+
+  g_return_val_if_fail (GDK_IS_WAYLAND_DISPLAY(display), NULL);
+
+  return wayland_display->wl_display;
+}
+
+/**
+ * gdk_wayland_display_get_wl_compositor
+ * @display: (type GdkWaylandDisplay): a #GdkDisplay
+ *
+ * Returns the Wayland global singleton compositor of a #GdkDisplay
+ *
+ * Returns: (transfer none): a Wayland wl_compositor
+ *
+ * Since: 3.8
+ */
+struct wl_compositor *
+gdk_wayland_display_get_wl_compositor (GdkDisplay *display)
+{
+  GdkWaylandDisplay *wayland_display = GDK_WAYLAND_DISPLAY(display);
+
+  g_return_val_if_fail (GDK_IS_WAYLAND_DISPLAY(display), NULL);
+
+  return wayland_display->compositor;
+}
+
+/**
+ * gdk_wayland_display_get_xdg_shell
+ * @display: (type GdkWaylandDisplay): a #GdkDisplay
+ *
+ * Returns the Wayland global singleton shell of a #GdkDisplay
+ *
+ * Returns: (transfer none): a Wayland xdg_shell
+ *
+ * Since: 3.8
+ */
+struct xdg_shell *
+gdk_wayland_display_get_xdg_shell (GdkDisplay *display)
+{
+  GdkWaylandDisplay *wayland_display = GDK_WAYLAND_DISPLAY(display);
+
+  g_return_val_if_fail (GDK_IS_WAYLAND_DISPLAY(display), NULL);
+
+  return wayland_display->xdg_shell;
 }
