@@ -26,7 +26,7 @@
  */
 
 /* Cannot use TrackMouseEvent, as the stupid WM_MOUSELEAVE message
- * doesn't tell us where the mouse has gone. Thus we cannot use it to
+ * doesn’t tell us where the mouse has gone. Thus we cannot use it to
  * generate a correct GdkNotifyType. Pity, as using TrackMouseEvent
  * otherwise would make it possible to reliably generate
  * GDK_LEAVE_NOTIFY events, which would help get rid of those pesky
@@ -1009,13 +1009,13 @@ apply_event_filters (GdkWindow  *window,
  * On Windows, transient windows will not have their own taskbar entries.
  * Because of this, we must hide and restore groups of transients in both
  * directions.  That is, all transient children must be hidden or restored
- * with this window, but if this window's transient owner also has a
- * transient owner then this window's transient owner must be hidden/restored
+ * with this window, but if this window’s transient owner also has a
+ * transient owner then this window’s transient owner must be hidden/restored
  * with this one.  And etc, up the chain until we hit an ancestor that has no
  * transient owner.
  *
- * It would be a good idea if applications don't chain transient windows
- * together.  There's a limit to how much evil GTK can try to shield you
+ * It would be a good idea if applications don’t chain transient windows
+ * together.  There’s a limit to how much evil GTK can try to shield you
  * from.
  */
 static void
@@ -3076,8 +3076,27 @@ gdk_event_translate (MSG  *msg,
 	}
       else
 	{
-	  mmi->ptMaxTrackSize.x = 30000;
-	  mmi->ptMaxTrackSize.y = 30000;
+	  HMONITOR winmon;
+	  MONITORINFO moninfo;
+
+	  winmon = MonitorFromWindow (GDK_WINDOW_HWND (window), MONITOR_DEFAULTTONEAREST);
+
+	  moninfo.cbSize = sizeof (moninfo);
+
+	  if (GetMonitorInfoA (winmon, &moninfo))
+	    {
+	      mmi->ptMaxTrackSize.x = moninfo.rcWork.right - moninfo.rcWork.left;
+	      mmi->ptMaxTrackSize.y = moninfo.rcWork.bottom - moninfo.rcWork.top;
+	    }
+	  else
+	    {
+	      /* Apparently, this is just a very big number (bigger than any widget
+               * could realistically be) to make sure the window is as big as
+               * possible when maximized.
+               */
+	      mmi->ptMaxTrackSize.x = 30000;
+	      mmi->ptMaxTrackSize.y = 30000;
+	    }
 	}
 
       if (impl->hint_flags & (GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE))
@@ -3328,7 +3347,7 @@ gdk_event_prepare (GSource *source,
   *timeout = -1;
 
   if (_gdk_display->event_pause_count > 0)
-    retval = FALSE;
+    retval =_gdk_event_queue_find_first (_gdk_display) != NULL;
   else
     retval = (_gdk_event_queue_find_first (_gdk_display) != NULL ||
               (modal_win32_dialog == NULL &&
@@ -3347,7 +3366,7 @@ gdk_event_check (GSource *source)
   gdk_threads_enter ();
 
   if (_gdk_display->event_pause_count > 0)
-    retval = FALSE;
+    retval = _gdk_event_queue_find_first (_gdk_display) != NULL;
   else if (event_poll_fd.revents & G_IO_IN)
     retval = (_gdk_event_queue_find_first (_gdk_display) != NULL ||
               (modal_win32_dialog == NULL &&
