@@ -796,16 +796,14 @@ gdk_win32_window_destroy (GdkWindow *window,
     }
 }
 
-static cairo_surface_t *
+static gboolean
 gdk_win32_window_resize_cairo_surface (GdkWindow       *window,
                                        cairo_surface_t *surface,
                                        gint             width,
                                        gint             height)
 {
   /* XXX: Make Cairo surface use DC clip */
-  cairo_surface_destroy (surface);
-
-  return NULL;
+  return FALSE;
 }
 
 static void
@@ -2022,7 +2020,7 @@ gdk_win32_window_get_geometry (GdkWindow *window,
     }
 }
 
-static gint
+static void
 gdk_win32_window_get_root_coords (GdkWindow *window,
 				  gint       x,
 				  gint       y,
@@ -2048,7 +2046,6 @@ gdk_win32_window_get_root_coords (GdkWindow *window,
 			   GDK_WINDOW_HWND (window),
 			   x, y,
 			   tx + _gdk_offset_x, ty + _gdk_offset_y));
-  return 1;
 }
 
 static void
@@ -2066,27 +2063,6 @@ gdk_win32_window_restack_toplevel (GdkWindow *window,
 	// ### TODO
 }
 
-static void
-gdk_win32_window_get_root_origin (GdkWindow *window,
-			    gint      *x,
-			    gint      *y)
-{
-  GdkRectangle rect;
-
-  g_return_if_fail (GDK_IS_WINDOW (window));
-
-  gdk_window_get_frame_extents (window, &rect);
-
-  if (x)
-    *x = rect.x;
-
-  if (y)
-    *y = rect.y;
-
-  GDK_NOTE (MISC, g_print ("gdk_window_get_root_origin: %p: %+d%+d\n",
-			   GDK_WINDOW_HWND (window), rect.x, rect.y));
-}
-
 static void
 gdk_win32_window_get_frame_extents (GdkWindow    *window,
                               GdkRectangle *rect)
@@ -3320,7 +3296,7 @@ gdk_win32_window_is_win32 (GdkWindow *window)
  * 
  * Gets a DC with the given drawable selected into it.
  *
- * Return value: The DC, on success. Otherwise
+ * Returns: The DC, on success. Otherwise
  *  %NULL. If this function succeeded
  *  _gdk_win32_impl_release_dc()  must be called
  *  release the DC when you are done using it.
@@ -3475,7 +3451,6 @@ gdk_window_impl_win32_class_init (GdkWindowImplWin32Class *klass)
   impl_class->set_role = gdk_win32_window_set_role;
   //impl_class->set_startup_id = gdk_x11_window_set_startup_id;
   impl_class->set_transient_for = gdk_win32_window_set_transient_for;
-  impl_class->get_root_origin = gdk_win32_window_get_root_origin;
   impl_class->get_frame_extents = gdk_win32_window_get_frame_extents;
   impl_class->set_override_redirect = gdk_win32_window_set_override_redirect;
   impl_class->set_accept_focus = gdk_win32_window_set_accept_focus;
