@@ -278,7 +278,7 @@ gdk_event_source_prepare (GSource *source,
   *timeout = -1;
 
   if (display->event_pause_count > 0)
-    retval = FALSE;
+    retval = _gdk_event_queue_find_first (display) != NULL;
   else
     retval = (_gdk_event_queue_find_first (display) != NULL ||
               gdk_check_xpending (display));
@@ -297,7 +297,7 @@ gdk_event_source_check (GSource *source)
   gdk_threads_enter ();
 
   if (event_source->display->event_pause_count > 0)
-    retval = FALSE;
+    retval = _gdk_event_queue_find_first (event_source->display) != NULL;
   else if (event_source->event_poll_fd.revents & G_IO_IN)
     retval = (_gdk_event_queue_find_first (event_source->display) != NULL ||
               gdk_check_xpending (event_source->display));
